/*NB_REVISION*/

/*NB_COPYRIGHT*/

/*------------------------------------------------------------------------------
 * EFFS-STD configuration file for 8MB SPI flash chip used on the NANO54415 and
 * SB800EX.
 *
 * To modify the amount of space allocated to the file system:
 *
 * 1. Change the definition in this file:  #define FS_SIZE ( 64 * 1024 )
 * 2. Change the compcode memory address range for the application in your
 *    NBEclipse project settings (or makefile) so that the end of the
 *    application space does not exceed the start of the file system space.
 *    Refer to the EFFS Programming Guide section of the manual for more details.
 * 3. Be sure to add the \Nburn\lib\StdFFile.a library to your NBEclipse project
 *    C/C++ build linker library options.  See the header comments in main.cpp
 *    for this example on how to add the library in the NBEclipse project.
 *----------------------------------------------------------------------------*/

#ifndef _ONCHIPFLASH_H_
#define _ONCHIPFLASH_H_

#include "basictypes.h"
#include "hal.h"
#include "file/fsf.h"

/**
 * Function implemented.
 */
extern int fs_phy_OnChipFlash(FS_FLASH *flash);

#define FLASH_NAME "MX25L6406E"

/**
 * BLOCKSIZE
 * This defines the size of the blocks to be used in the file storage area.
 * This must be an erasable unit of the flash chip.  All blocks in the file
 * storage area must be the same size.  This may be different from the DESCSIZE
 * where the flash chip has different size erasable units available.
 *
 * SECTORSIZE
 * This defines the sector size.  Each block is divided into a number of
 * sectors.  This number is the smallest usable unit in the system and thus
 * represents the minimum file storage area.  For best usage of the flash
 * blocks, the sector size should always be a power of two.  See sector section
 * below for more information.
 *
 * SECTORPERBLOCK
 * This defines the number of sectors in a block.  It must always be true that:
 *
 *    SECTORPERBLOCK = BLOCKSIZE / SECTORSIZE
 *
 * The memory map below is for a module with a an 8MB SPI flash with a 4k sector size.
 * This example will allocate 1MB for the EFFS STD flash file system. There are a total
 * of 2048 4k sectors.
 *
 *                            Address In SPI Flash
 *                            -------
 *    ----------------------  0080 0000  (End of flash space)
 *    |  File System Data  |
 *    |  1MB - 8k          |
 *    |  4k x 254 Blocks   |
 *    |--------------------|  0070 2000 (Start of file system data)
 *    |  DESC BLOCKS 0/1   |
 *    |  8k                |
 *    |  4k x 2 Blocks     |
 *    |--------------------|  0070 0000 (Start of file system)
 *    |                    |
 *    |  Application       |
 *    |  7.3MB             |
 *    |  4k x 1786 Blocks  |
 *    |                    |
 *    |--------------------|  0000 4000
 *    |  8k User Params    |
 *    |  4k x 2 Blocks     |
 *    |--------------------|  0000 2000
 *    |  8k Global config  |
 *    |  4k x 4 Blocks     |
 *    |--------------------|  0000 0000 (Start of flash space)
 */

/**
 * Start of flash memory base address.
 */
//#define FS_FLASHBASE   ( 0x00000000 )

/*
 * CHANGES TO COMPCODE FLAGS
 * In NBEclipse, or your command line makefile, change the following line
 * so the application will only occupy the specified application space.
 * The first parameter is the start of application space, and the second
 * is the address just below the file system space.
 *
 *    COMPCODEFLAGS = 0x04000 0x800000      // Original
 *
 *    COMPCODEFLAGS = 0x04000 0x700000      // space for file system
 */

#define BLOCKSIZE (4 * 1024)   // All sectors are 4KB
#define SECTORSIZE (512)       // 8 sectors per block
#define SECTORPERBLOCK (BLOCKSIZE / SECTORSIZE)

/*
 * Specify the total amount of flash memory in the system, and the amount
 * allocated to be used by the file system (the rest is used by the
 * application).
 */
/* Example for a 64k file system */
//#define FLASH_SIZE   (        8*1024 * 1024 )   // Total flash space
//#define FS_SIZE      (            64 * 1024 )   // Amount of Flash allocated to file system
//#define FIRST_ADDR   ( FLASH_SIZE - FS_SIZE )   // First Flash file system address
//#define BLOCKSTART   (                    2 )   // First block where file system data starts (first 2 blocks are DESCRIPTORS)

/* Example for a 1MB file system */
#define FLASH_SIZE (8 * 1024 * 1024)        // Total flash space
#define FS_SIZE (1 * 1024 * 1024)           // Amount of Flash allocated to file system
#define FIRST_ADDR (FLASH_SIZE - FS_SIZE)   // First Flash file system address
#define BLOCKSTART (16)                     // First block where file system data starts, after blocks reserved for Descriptors

/*
 * Descriptor Blocks:
 * These blocks contain critical information about the file system, block
 * allocation, wear information, and file/directory information.  At least two
 * descriptor blocks must be included in the system, which can be erased
 * independently.  An optional descriptor write cache may be configured which
 * improves the performance of the file system.  Please refer to the EFFS-STD
 * implementation guide for additional information.
 */
/* Example for a 64k file system */
//#define DESCSIZE         ( 4 * 1024 )   // Size of one descriptor
//#define DESCBLOCKSTART   (        0 )   // Position of first descriptor
//#define DESCBLOCKEND     (        1 )   // Position of last descriptor
//#define DESCCACHE        (     1024 )

/* Example for a 1MB file system */
#define DESCSIZE (64 * 1024)   // Size of one descriptor, 8 BLOCKS
#define DESCBLOCKSTART (0)     // Position of first descriptor
#define DESCBLOCKEND (1)       // Position of last descriptor
#define DESCCACHE (1024)

#endif /* _ONCHIPFLASH_H_ */
