/*NB_REVISION*/

/*NB_COPYRIGHT*/

#include <hal.h>
#include <init.h>
#include <nbtime.h>
#include <netbios.h>
#include <netinterface.h>

// NB SSL library
#include <crypto/ssl.h>

//#include <syslog.h>
#define SysLog(...) (void)0;

// Product Definitions
#include "nbfactory.h"

// Ethernet to Serial Application Definitions
#include "serialburnerdata.h"

// SSL User Routines
#include "ssluser.h"

// NB EFFS-STD library
#include "effs_std.h"
#include "effs_time.h"

void ScanCaCerts();
bool ClearCaList();

extern const char *default_page;   // Default HTTP startup page
extern void DisplayNvSettings(void);

const char FirmwareVersion[] = NB_FACTORY_MODULE_BASE_NAME NB_FACTORY_FEATURE_NAME __DATE__ " OS:" NB_VERSION_TEXT " ";
BOOL bShowDebug = 0;   // Debug display flag

extern MonitorRecord monitor_config;

/*-----------------------------------------------------------------------------
 * Display debug commands
 *----------------------------------------------------------------------------*/
void DisplayDebugMenu()
{
    iprintf("\r\nDebug commands :\r\n");
    iprintf("0 Debug messages off\r\n");
    iprintf("1 Debug messages on\r\n");
    iprintf("D reset to factory default parameters\r\n");
    iprintf("F show file system info\r\n");
    iprintf("G format file system - caution!\r\n");
    iprintf("N display NV Settings\r\n");
    iprintf("T display system time\r\n");
}

/*-----------------------------------------------------------------------------
 * Process debug commands
 *----------------------------------------------------------------------------*/

void ProcessDebugCommand(char c)
{
    switch (toupper(c))
    {
        case '0':
            bShowDebug = FALSE;
            debug_iprintf("Debug is off\r\n");
            break;
        case '1':
            bShowDebug = TRUE;
            debug_iprintf("Debug is on\r\n");
            break;
        case 'B': ForceReboot(); break;
        case 'D':
            debug_iprintf("Setting default values\r\n");
            CheckNVSettings(true);
            break;
        case 'F':
            EffsListCurrentDirectory(NV_Settings.DeviceName);
            EffsDisplayStatistics(NV_Settings.DeviceName);
            break;
        case 'G':
        {
            if(EffsFormat() == FS_NOERR)
            {
                // If we've wiped the file system, we need to make corresponding adjustments to
                // what certs are being referenced in the NVSettings, otherwise everything will
                // get out of synch.
                if(NV_Settings.SslCertificateSource != SERIAL_BURNER_LIBRARY_DEFAULT)
                {
                    NV_Settings.SslCertificateSource = SERIAL_BURNER_LIBRARY_DEFAULT;
                    NV_Settings.SslCertificateLength = 0;
                    NV_Settings.SslKeyLength = 0;
                    SaveUserParameters(&NV_Settings, sizeof(NV_Settings));
                }
            }
            break;
        }
        case 'N': DisplayNvSettings(); break;
        case 'T':
            DisplaySystemTime();   // Display current system date and time
            iprintf("\r\n");
            break;
        default: DisplayDebugMenu();
    }
}

// Application name
const char *AppName = NB_FACTORY_MODULE_BASE_NAME NB_FACTORY_FEATURE_NAME " " NB_FACTORY_FEATURE_DESC;

/*-----------------------------------------------------------------------------
 * Main task
 *-----------------------------------------------------------------------------*/
void UserMain(void *notUsedPtr)
{
    init();
    //Enable system diagnostics. Progably should remove for production code.
    EnableSystemDiagnostics(); 
    WaitForActiveNetwork(TICKS_PER_SECOND * 5);   // Wait for DHCP address

    // EFFS-STD
    EffsStart((char *)"EffsStart");

    CheckNVSettings();

    bShowDebug = TRUE;

    if (SetTimeNTP())
    {
        // tzsetchar((char*)"EST5EDT4,M3.2.0/01:00:00,M11.1.0/02:00:00");
        // tzsetchar((char*)"CST6CDT5,M3.2.0/01:00:00,M11.1.0/02:00:00");
        // tzsetchar((char*)"MST7MDT6,M3.2.0/01:00:00,M11.1.0/02:00:00");
        tzsetchar((char *)"PST8PDT7,M3.2.0/01:00:00,M11.1.0/02:00:00");
    }
    else
    {
        iprintf("NTP failed, setting time manually\r\n");
    }
    DisplaySystemTime();
    iprintf("\r\n");

    // Start SSL service
    // This is only required because we will be adding certificates to the CA list one at time before a connection is made. Otherwise
    // a call to SSL_connect() would take care of everything.
    SslInit();

    // Scan the CA List - This needs to happen after the system time has been established, otherwise you will get errors
    // trying to load the certs
    ScanCaCerts();

    SysLog("Starting up\r\n");

    StartHttps();

    default_page = "https.html";

    // Publish NetBIOS name
    NetbiosEnableNameService(NV_Settings.NetBIOSName, TRUE);

    // If we don't have any certs loaded already, load up the compiled in ones
    if (NV_Settings.SslCertificateSource == SERIAL_BURNER_LIBRARY_DEFAULT)
    {
        iprintf("Setting SSL/TLS defaults\r\n");
        SslUserSetDefault();
        SaveUserParameters(&NV_Settings, sizeof(NV_Settings));
    }

    // Display product name
    if (!monitor_config.Quiet)
    {
        iprintf("%s : ", NV_Settings.DeviceName);
        iprintf("Product->    %s%s", NB_FACTORY_MODULE_BASE_NAME, NB_FACTORY_FEATURE_NAME);
        iprintf(" %s\r\n", NB_FACTORY_FEATURE_DESC);
    }

    boot_iprintf("Device->     %s", NV_Settings.DeviceName);
    InterfaceBlock *pifb = GetInterfaceBlock();
    if ((pifb != nullptr) && (!monitor_config.Quiet == 0))
    {
        iprintf("%s : ", NV_Settings.DeviceName);
        iprintf("IP Address-> %hI\r\n", pifb->ip4.cur_addr.i4);
    }

    boot_iprintf("Version->    %s", NB_FACTORY_DEFAULTS_VERSION_STRING " Created:" __DATE__);
    debug_iprintf("Settings size->    %ld", sizeof(NV_Settings));
    boot_iprintf("? for debug commands");
    OSTimeDly(TICKS_PER_SECOND / 2);

    // EFFS-STD
    // EffsListCurrentDirectory(NV_Settings.DeviceName);
    // EffsDisplayStatistics(NV_Settings.DeviceName);

    while (1)
    {
        char c = getchar();
        ProcessDebugCommand(c);
    }
}
